from __future__ import annotations

from typing import Any

from .FXApp import FXApp
from .FXColor import FXColor
from .FXDrawable import FXDrawable

#: Keep pixel data in client.
IMAGE_KEEP: int = hash("IMAGE_KEEP")

#: Pixel data is owned by image.
IMAGE_OWNED: int = hash("IMAGE_OWNED")

#: Dither image to look better.
IMAGE_DITHER: int = hash("IMAGE_DITHER")

#: Turn off dithering and map to nearest color.
IMAGE_NEAREST: int = hash("IMAGE_NEAREST")

#: Data has alpha channel.
IMAGE_ALPHA: int = hash("IMAGE_ALPHA")

#: Force opaque background.
IMAGE_OPAQUE: int = hash("IMAGE_OPAQUE")

#: Override transparancy color.
IMAGE_ALPHACOLOR: int = hash("IMAGE_ALPHACOLOR")

#: Using shared memory image.
IMAGE_SHMI: int = hash("IMAGE_SHMI")

#: Using shared memory pixmap.
IMAGE_SHMP: int = hash("IMAGE_SHMP")

#: Guess transparency color from corners.
IMAGE_ALPHAGUESS: int = hash("IMAGE_ALPHAGUESS")


class FXImage(FXDrawable):
    """Image class."""

    def __init__(self, a: FXApp, pix: Any | None = None, opts: int = 0, w: int = 1, h: int = 1):
        """Create an image.

        Parameters
        ----------
        a : FXApp

        pix : Any | None

        opts : int

        w : int

        h : int
        """

    def blend(self, color: FXColor, sharpen: bool = True):
        """Blends the icon with the specified color; should only be used on icons that support an alpha channel,
        for example, PNG.

        Parameters
        ----------
        color : FXColor

        sharpen : bool
        """

    def create(self):
        """Create image resource.

        Reimplemented from FXId. Reimplemented in FXIcon.
        """

    def destroy(self):
        """Destroy image resource.

        Reimplemented from FXId. Reimplemented in FXIcon.
        """

    def detach(self):
        """Detach image resource.

        Reimplemented from FXId. Reimplemented in FXIcon.
        """

    def getOptions(self):
        """To get to the option flags."""

    def getPixel(self, x: int, y: int):
        """Get pixel at x,y.

        Parameters
        ----------
        x : int

        y : int
        """

    def render(self):
        """Render the image from client-side pixel buffer.

        Reimplemented in FXIcon.
        """

    def resize(self, w: int, h: int):
        """Resize pixmap to the specified width and height. Reimplemented from FXDrawable. Reimplemented in
        FXIcon.

        Parameters
        ----------
        w : int

        h : int
        """

    def scale(self, w: int, h: int):
        """Rescale pixels image to the specified width and height.

        Parameters
        ----------
        w : int

        h : int
        """

    def setPixel(self, x: int, y: int, color: FXColor):
        """Change pixel at x,y.

        Parameters
        ----------
        x : int

        y : int

        color : FXColor
        """



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .._OptionsBase import _OptionsBase
from ..UtilityAndView.abaqusConstants import (
    MODEL_SIZE,
    OFF,
    ON,
    RESULTANT,
    SCIENTIFIC,
    Boolean,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class FreeBodyOptions(_OptionsBase):
    """The FreeBodyOptions object stores values and attributes associated with a free body
    plot. The FreeBodyOptions object has no constructor command. Abaqus creates a
    *defaultOdbDisplay.freeBodyOptions* member when you import the Visualization module.
    Abaqus creates a **FreeBodyOptions** member when it creates the OdbDisplay object, using
    the values from *defaultOdbDisplay.freeBodyOptions*. Abaqus creates the **odbDisplay**
    member when a viewport is created, using the values from **defaultOdbDisplay**.
    FreeBodyOptions objects are accessed in one of two ways:
    - The default free body options. These settings are used as defaults when other
    **freeBodyOptions** members are created. These settings can be set to customize user
    preferences.
    - The free body options associated with a particular viewport.

    .. note::
        This object can be accessed by::

            import visualization
            session.defaultOdbDisplay.freeBodyOptions
            session.viewports[name].layers[name].odbDisplay.freeBodyOptions
            session.viewports[name].odbDisplay.freeBodyOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        *,
        comp1ColorF: str = "",
        comp1ColorM: str = "",
        comp2ColorF: str = "",
        comp2ColorM: str = "",
        comp3ColorF: str = "",
        comp3ColorM: str = "",
        resultantColorF: str = "",
        resultantColorM: str = "",
        textColorF: str = "",
        textColorM: str = "",
        textFontF: str = "",
        textFontM: str = "",
        numberFormatF: Literal[C.SCIENTIFIC, C.ENGINEERING, C.FIXED] = SCIENTIFIC,
        numberFormatM: Literal[C.SCIENTIFIC, C.ENGINEERING, C.FIXED] = SCIENTIFIC,
        scaleModeF: Literal[C.SCREEN_SIZE, C.MODEL_SIZE] = MODEL_SIZE,
        scaleModeM: Literal[C.SCREEN_SIZE, C.MODEL_SIZE] = MODEL_SIZE,
        vectorDisplay: Literal[C.RESULTANT, C.COMPONENT] = RESULTANT,
        numDigitsF: int = 3,
        numDigitsM: int = 3,
        sizePercentageF: float = 10,
        sizePercentageM: float = 10,
        thresholdF: float | None = None,
        thresholdM: float | None = None,
        drawLabelF: Boolean = ON,
        drawLabelM: Boolean = ON,
        showComp1F: Boolean = ON,
        showComp1M: Boolean = ON,
        showComp2F: Boolean = ON,
        showComp2M: Boolean = ON,
        showComp3F: Boolean = ON,
        showComp3M: Boolean = ON,
        showForce: Boolean = ON,
        showMoment: Boolean = ON,
        constantLengthArrow: Boolean = OFF,
    ):
        """This method modifies the FreeBodyOptions object.

        Parameters
        ----------
        comp1ColorF
            A String specifying color of the first force component. The default value is "#FF1932".
        comp1ColorM
            A String specifying color of the first moment component. The default value is "#0000FF".
        comp2ColorF
            A String specifying color of the second force component. The default value is "#FF1932".
        comp2ColorM
            A String specifying color of the second moment component. The default value is
            "#0000FF".
        comp3ColorF
            A String specifying color of the third force component. The default value is "#FF1932".
        comp3ColorM
            A String specifying color of the third moment component. The default value is "#0000FF".
        resultantColorF
            A String specifying color of the resultant force. The default value is "#FF1932".
        resultantColorM
            A String specifying color of the resultant moment. The default value is "#0000FF".
        textColorF
            A String specifying text color for force. The default value is "Yellow".
        textColorM
            A String specifying text color for moment. The default value is "Yellow".
        textFontF
            A String specifying text font for force. The default value is "verdana".
        textFontM
            A String specifying text font for moment. The default value is "verdana".
        numberFormatF
            A SymbolicConstant specifying the number format for force. Possible values are
            SCIENTIFIC, FIXED, and ENGINEERING. The default value is SCIENTIFIC.
        numberFormatM
            A SymbolicConstant specifying the number format for moment. Possible values are
            SCIENTIFIC, FIXED, and ENGINEERING. The default value is SCIENTIFIC.
        scaleModeF
            A SymbolicConstant specifying the size scaling mode for force. Possible values are
            MODEL_SIZE and SCREEN_SIZE. The default value is MODEL_SIZE.
        scaleModeM
            A SymbolicConstant specifying the size scaling mode for moment. Possible values are
            MODEL_SIZE and SCREEN_SIZE. The default value is MODEL_SIZE.
        vectorDisplay
            A SymbolicConstant specifying the vector display mode. Possible values are RESULTANT and
            COMPONENT. The default value is RESULTANT.
        numDigitsF
            An Int specifying the number of digits in the force label. The default value is 3.
        numDigitsM
            An Int specifying the number of digits in the moment label. The default value is 3.
        sizePercentageF
            A Float specifying the size of the force symbol as a percentage of the screen or model.
            The default value is 10.0.
        sizePercentageM
            A Float specifying the size of the moment symbol as a percentage of the screen or model.
            The default value is 10.0.
        thresholdF
            A Float specifying the force threshold value. The default value is 10⁻⁶.
        thresholdM
            A Float specifying the moment threshold value. The default value is 10⁻⁶.
        drawLabelF
            A Boolean specifying whether to draw force labels. The default value is ON.
        drawLabelM
            A Boolean specifying whether to draw moment labels. The default value is ON.
        showComp1F
            A Boolean specifying whether to show the first force component. The default value is ON.
        showComp1M
            A Boolean specifying whether to show the first moment component. The default value is
            ON.
        showComp2F
            A Boolean specifying whether to show the second force component. The default value is
            ON.
        showComp2M
            A Boolean specifying whether to show the second moment component. The default value is
            ON.
        showComp3F
            A Boolean specifying whether to show the third force component. The default value is ON.
        showComp3M
            A Boolean specifying whether to show the third moment component. The default value is
            ON.
        showForce
            A Boolean specifying whether to show forces. The default value is ON.
        showMoment
            A Boolean specifying whether to show moments. The default value is ON.
        constantLengthArrow
            A Boolean specifying whether to use a constant length for all arrows. The default value
            is OFF.
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import ISOTROPIC, OFF, Boolean
from ...UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Conductivity:
    r"""The Conductivity object specifies thermal conductivity.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].conductivity
            import odbMaterial
            session.odbs[name].materials[name].conductivity

        The table data for this object are:

        - If **type** = ISOTROPIC, the table data specify the following:

            - Conductivity, :math:`k`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = ORTHOTROPIC, the table data specify the following:

            - :math:`k_{11}`
            - :math:`k_{22}`.
            - :math:`k_{33}`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = ANISOTROPIC, the table data specify the following:

            - :math:`k_{11}`.
            - :math:`k_{12}`.
            - :math:`k_{22}`.
            - :math:`k_{13}`.
            - :math:`k_{23}`.
            - :math:`k_{33}`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - CONDUCTIVITY
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        type: Literal[C.ANISOTROPIC, C.ISOTROPIC, C.ORTHOTROPIC] = ISOTROPIC,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a Conductivity object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Conductivity
                session.odbs[name].materials[name].Conductivity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        type
            A SymbolicConstant specifying the type of conductivity. Possible values are ISOTROPIC,
            ORTHOTROPIC, and ANISOTROPIC. The default value is ISOTROPIC.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        Conductivity
            A Conductivity object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Conductivity object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class ContactStabilization:
    """The ContactStabilization object is the abstract base type for other ContactStabilization objects. The
    ContactStabilization object has no explicit constructor, members, or methods.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].contactStabilizations[name]
    """

    ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Step.StepBase import StepBase
from ..UtilityAndView.abaqusConstants import ON, Boolean
from .LoadCase import LoadCase


@abaqus_class_doc
class LoadStep(StepBase):
    @abaqus_method_doc
    def LoadCase(
        self,
        name: str,
        boundaryConditions: tuple = (),
        loads: tuple = (),
        includeActiveBaseStateBC: Boolean = ON,
    ) -> LoadCase:
        """This method creates a load case in a step.

        .. note::
            This function can be accessed by::

                mdb.models[name].steps[name].LoadCase

        Parameters
        ----------
        name
            A String specifying the name of the object.
        boundaryConditions
            A sequence of (String, Float) sequences specifying the name of a BoundaryCondition
            followed by a nonzero Float scaling factor. The default value is an empty sequence.
        loads
            A sequence of (String, Float) sequences specifying the name of a Load followed by a
            nonzero Float specifying a scale factor. The default value is an empty sequence.
        includeActiveBaseStateBC
            A Boolean specifying whether to include all active boundary conditions propagated or
            modified from the base state. The default value is ON.

        Returns
        -------
        LoadCase
            A LoadCase object.

        Raises
        ------
        RangeError
        """
        self.loadCases[name] = loadCase = LoadCase(name, boundaryConditions, loads, includeActiveBaseStateBC)
        return loadCase



from __future__ import annotations

from typing import Sequence, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .._OptionsBase import _OptionsBase
from ..UtilityAndView.abaqusConstants import (
    BOTTOM_LEFT,
    FILLED_ARROW,
    NONE,
    SOLID,
    VERY_THIN,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Annotation import Annotation


@abaqus_class_doc
class Arrow(Annotation, _OptionsBase):
    """The Arrow object stores the visual settings and location of an arrow annotation. The Arrow object is
    derived from the Annotation object.

    .. note::
        This object can be accessed by::

            import annotationToolset
            mdb.annotations[name]
            session.odbs[name].userData.annotations[name]
            session.viewports[name].annotationsToPlot[i]
    """

    #: A String specifying the annotation repository key.
    name: str

    #: A pair of Floats specifying the start point **X**  and **Y** offsets in millimeters from
    #: **startAnchor**. The default value is (0, 0).
    startPoint: Sequence[float] = (0.0, 0.0)

    #: A pair of Floats specifying the end point **X**  and **Y** offsets in millimeters from
    #: **endAnchor**. The default value is (0, 0).
    endPoint: Sequence[float] = (0.0, 0.0)

    #: A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
    #: specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
    #: A sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
    #: the model coordinate system. A SymbolicConstant indicates a relative position. Possible
    #: values are:
    #:
    #: - BOTTOM_LEFT
    #: - BOTTOM_CENTER
    #: - BOTTOM_RIGHT
    #: - CENTER_LEFT
    #: - CENTER
    #: - CENTER_RIGHT
    #: - TOP_LEFT
    #: - TOP_CENTER
    #: - TOP_RIGHT
    #:
    #: The default value is BOTTOM_LEFT.
    startAnchor: Union[
        Literal[
            C.CENTER_RIGHT,
            C.TOP_CENTER,
            C.BOTTOM_RIGHT,
            C.BOTTOM_LEFT,
            C.CENTER,
            C.TOP_RIGHT,
            C.CENTER_LEFT,
            C.TOP_LEFT,
            C.BOTTOM_CENTER,
        ],
        tuple[float, ...],
    ] = BOTTOM_LEFT

    #: A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
    #: specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
    #: A Sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
    #: the model coordinate system. A SymbolicConstant indicates a relative position. Possible
    #: values are:
    #:
    #: - BOTTOM_LEFT
    #: - BOTTOM_CENTER
    #: - BOTTOM_RIGHT
    #: - CENTER_LEFT
    #: - CENTER
    #: - CENTER_RIGHT
    #: - TOP_LEFT
    #: - TOP_CENTER
    #: - TOP_RIGHT
    #:
    #: The default value is BOTTOM_LEFT.
    endAnchor: Union[
        Literal[
            C.CENTER_RIGHT,
            C.TOP_CENTER,
            C.BOTTOM_RIGHT,
            C.BOTTOM_LEFT,
            C.CENTER,
            C.TOP_RIGHT,
            C.CENTER_LEFT,
            C.TOP_LEFT,
            C.BOTTOM_CENTER,
        ],
        tuple[float, ...],
    ] = BOTTOM_LEFT

    #: A SymbolicConstant specifying the style of the start head. Possible values are:
    #:
    #: - ARROW
    #: - FILLED_ARROW
    #: - HOLLOW_CIRCLE
    #: - FILLED_CIRCLE
    #: - HOLLOW_DIAMOND
    #: - FILLED_DIAMOND
    #: - HOLLOW_SQUARE
    #: - FILLED_SQUARE
    #: - NONE
    #:
    #: The default value is NONE.
    startHeadStyle: SymbolicConstant = NONE

    #: A SymbolicConstant specifying the style of the end head. Possible values are:
    #:
    #: - ARROW
    #: - FILLED_ARROW
    #: - HOLLOW_CIRCLE
    #: - FILLED_CIRCLE
    #: - HOLLOW_DIAMOND
    #: - FILLED_DIAMOND
    #: - HOLLOW_SQUARE
    #: - FILLED_SQUARE
    #: - NONE
    #:
    #: The default value is FILLED_ARROW.
    endHeadStyle: SymbolicConstant = FILLED_ARROW

    #: A Float specifying the distance in millimeters between the arrow start point and the
    #: arrow start head. The default value is 0.0.
    startGap: float = 0.0

    #: A Float specifying the distance in millimeters between the arrow end point and the arrow
    #: end head. The default value is 0.0.
    endGap: float = 0.0

    #: A String specifying the color of the arrow. Possible string values are any valid color.
    #: The default value is "White".
    color: str = "White"

    #: A SymbolicConstant specifying the line style of the arrow. Possible values are SOLID,
    #: DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
    lineStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the line thickness of the arrow. Possible values are
    #: VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    lineThickness: SymbolicConstant = VERY_THIN

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        startPoint: Sequence[float] = (0.0, 0.0),
        endPoint: Sequence[float] = (0.0, 0.0),
        startAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            tuple[float, ...],
        ] = BOTTOM_LEFT,
        endAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            tuple[float, ...],
        ] = BOTTOM_LEFT,
        startHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = NONE,
        endHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = FILLED_ARROW,
        startGap: float = 0.0,
        endGap: float = 0.0,
        color: str = "White",
        lineStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        lineThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
    ):
        """This method creates an Arrow object.

        .. note::
            This function can be accessed by::

                mdb.Arrow
                session.odbs[name].userData.Arrow

        Parameters
        ----------
        name
            A String specifying the annotation repository key.
        startPoint
            A pair of Floats specifying the start point **X**  and **Y** offsets in millimeters from
            **startAnchor**. The default value is (0, 0).
        endPoint
            A pair of Floats specifying the end point **X**  and **Y** offsets in millimeters from
            **endAnchor**. The default value is (0, 0).
        startAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        endAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        startHeadStyle
            A SymbolicConstant specifying the style of the start head. Possible values are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is NONE.
        endHeadStyle
            A SymbolicConstant specifying the style of the end head. Possible values are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is FILLED_ARROW.
        startGap
            A Float specifying the distance in millimeters between the arrow start point and the
            arrow start head. The default value is 0.0.
        endGap
            A Float specifying the distance in millimeters between the arrow end point and the arrow
            end head. The default value is 0.0.
        color
            A String specifying the color of the arrow. Possible string values are any valid color.
            The default value is "White".
        lineStyle
            A SymbolicConstant specifying the line style of the arrow. Possible values are SOLID,
            DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
        lineThickness
            A SymbolicConstant specifying the line thickness of the arrow. Possible values are
            VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.

        Returns
        -------
        Arrow
            An Arrow object.
        """
        super().__init__()
        self.name = name
        self.startPoint = startPoint
        self.endPoint = endPoint
        self.startAnchor = startAnchor
        self.endAnchor = endAnchor
        self.startHeadStyle = startHeadStyle
        self.endHeadStyle = endHeadStyle
        self.startGap = startGap
        self.endGap = endGap
        self.color = color
        self.lineStyle = lineStyle
        self.lineThickness = lineThickness

    @abaqus_method_doc
    def translateStartPoint(self, x: float | None = None, y: float | None = None):
        """This method translates the start point of the Arrow object on the viewport plane.

        Parameters
        ----------
        x
            A Float specifying the **X** translation amount in millimeters.
        y
            A Float specifying the*Y* translation amount in millimeters.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def translateEndPoint(self, x: float | None = None, y: float | None = None):
        """This method translates the end point of the Arrow object on the viewport plane.

        Parameters
        ----------
        x
            A Float specifying the **X** translation amount in millimeters.
        y
            A Float specifying the*Y* translation amount in millimeters.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def setValues(
        self,
        startPoint: Sequence[float] = (0.0, 0.0),
        endPoint: Sequence[float] = (0.0, 0.0),
        startAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        endAnchor: Union[
            Literal[
                C.CENTER_RIGHT,
                C.TOP_CENTER,
                C.BOTTOM_RIGHT,
                C.BOTTOM_LEFT,
                C.CENTER,
                C.TOP_RIGHT,
                C.CENTER_LEFT,
                C.TOP_LEFT,
                C.BOTTOM_CENTER,
            ],
            float,
        ] = BOTTOM_LEFT,
        startHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = NONE,
        endHeadStyle: Literal[
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_SQUARE,
            C.FILLED_ARROW,
            C.HOLLOW_DIAMOND,
            C.ARROW,
            C.NONE,
            C.FILLED_CIRCLE,
        ] = FILLED_ARROW,
        startGap: float = 0.0,
        endGap: float = 0.0,
        color: str = "White",
        lineStyle: Literal[C.DASHED, C.SOLID, C.DOT_DASH, C.DOTTED] = SOLID,
        lineThickness: Literal[C.THIN, C.THICK, C.VERY_THIN, C.MEDIUM] = VERY_THIN,
    ):
        """This method modifies the Arrow object.

        Parameters
        ----------
        startPoint
            A pair of Floats specifying the start point **X**  and **Y** offsets in millimeters from
            **startAnchor**. The default value is (0, 0).
        endPoint
            A pair of Floats specifying the end point **X**  and **Y** offsets in millimeters from
            **endAnchor**. The default value is (0, 0).
        startAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values
            are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The
            default value is BOTTOM_LEFT.
        endAnchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**  and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**, **Y**, and **Z** coordinates of a point in
            the model coordinate system. A SymbolicConstant indicates a relative position. Possible
            values
            are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The
            default value is BOTTOM_LEFT.
        startHeadStyle
            A SymbolicConstant specifying the style of the start head. Possible values
            are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is NONE.
        endHeadStyle
            A SymbolicConstant specifying the style of the end head. Possible values
            are:

            - ARROW
            - FILLED_ARROW
            - HOLLOW_CIRCLE
            - FILLED_CIRCLE
            - HOLLOW_DIAMOND
            - FILLED_DIAMOND
            - HOLLOW_SQUARE
            - FILLED_SQUARE
            - NONE

            The default value is FILLED_ARROW.
        startGap
            A Float specifying the distance in millimeters between the arrow start point and the
            arrow start head. The default value is 0.0.
        endGap
            A Float specifying the distance in millimeters between the arrow end point and the arrow
            end head. The default value is 0.0.
        color
            A String specifying the color of the arrow. Possible string values are any valid color.
            The default value is "White".
        lineStyle
            A SymbolicConstant specifying the line style of the arrow. Possible values are SOLID,
            DASHED, DOTTED, and DOT_DASH. The default value is SOLID.
        lineThickness
            A SymbolicConstant specifying the line thickness of the arrow. Possible values are
            VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
        """
        super().setValues(
            startPoint=startPoint,
            endPoint=endPoint,
            startAnchor=startAnchor,
            endAnchor=endAnchor,
            startHeadStyle=startHeadStyle,
            endHeadStyle=endHeadStyle,
            startGap=startGap,
            endGap=endGap,
            color=color,
            lineStyle=lineStyle,
            lineThickness=lineThickness,
        )



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class LayerProperties:
    """The LayerProperties object defines the properties of a layer of reinforcement for membrane, shell, and
    surface sections.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].parts[name].compositeLayups[i].section.rebarLayers.layerTable[i]
            mdb.models[name].sections[name].rebarLayers.layerTable[i]
            import odbSection
            session.odbs[name].sections[name].rebarLayers.layerTable[i]

        The corresponding analysis keywords are:

        - REBAR LAYER
    """

    #: A Float specifying the area per bar.
    barArea: float

    #: A Float or a String specifying the orientation of the rebar. A Float specifies the
    #: angular orientation; a String specifies an orientation name.
    orientationAngle: float

    #: A String specifying the name of the rebar layer.
    layerName: str

    #: A String specifying the name of the rebar material.
    material: str

    #: A Float specifying the spacing of the rebar. This argument is only valid if the
    #: **rebarSpacing** argument on the parent RebarLayers object is set to CONSTANT. The default
    #: value is 0.0.
    barSpacing: float = 0

    #: A Float specifying the position of the rebar from the middle surface of the shell.
    #: **layerPosition** applies only for homogeneous shell sections and composite shell
    #: sections. The default value is 0.0.
    layerPosition: float = 0

    #: A Float specifying the spacing angle of the rebar. This argument is only valid if the
    #: **rebarSpacing** argument on the parent RebarLayers object is set to ANGULAR. The default
    #: value is 0.0.
    spacingAngle: float = 0

    #: A Float specifying the extension ratio for the rebar. This argument is only valid if the
    #: **rebarSpacing** argument on the parent RebarLayers object is set to LIFT_EQUATION. The
    #: default value is 0.0.
    extensionRatio: float = 0

    #: A Float specifying the radius of the rebar. This argument is only valid if the
    #: **rebarSpacing** argument on the parent RebarLayers object is set to LIFT_EQUATION. The
    #: default value is 0.0.
    radius: float = 0

    @abaqus_method_doc
    def __init__(
        self,
        barArea: float,
        orientationAngle: float,
        layerName: str,
        material: str,
        barSpacing: float = 0,
        layerPosition: float = 0,
        spacingAngle: float = 0,
        extensionRatio: float = 0,
        radius: float = 0,
    ):
        """This method creates a LayerProperties object.

        .. note::
            This function can be accessed by::

                section.LayerProperties
                odbSection.LayerProperties

        Parameters
        ----------
        barArea
            A Float specifying the area per bar.
        orientationAngle
            A Float or a String specifying the orientation of the rebar. A Float specifies the
            angular orientation; a String specifies an orientation name.
        layerName
            A String specifying the name of the rebar layer.
        material
            A String specifying the name of the rebar material.
        barSpacing
            A Float specifying the spacing of the rebar. This argument is only valid if the
            **rebarSpacing** argument on the parent RebarLayers object is set to CONSTANT. The default
            value is 0.0.
        layerPosition
            A Float specifying the position of the rebar from the middle surface of the shell.
            **layerPosition** applies only for homogeneous shell sections and composite shell
            sections. The default value is 0.0.
        spacingAngle
            A Float specifying the spacing angle of the rebar. This argument is only valid if the
            **rebarSpacing** argument on the parent RebarLayers object is set to ANGULAR. The default
            value is 0.0.
        extensionRatio
            A Float specifying the extension ratio for the rebar. This argument is only valid if the
            **rebarSpacing** argument on the parent RebarLayers object is set to LIFT_EQUATION. The
            default value is 0.0.
        radius
            A Float specifying the radius of the rebar. This argument is only valid if the
            **rebarSpacing** argument on the parent RebarLayers object is set to LIFT_EQUATION. The
            default value is 0.0.

        Returns
        -------
        LayerProperties
            A LayerProperties object.
        """
        ...



from __future__ import annotations

from typing import List

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class OdbFieldVarList(List[str]):
    """The read-only OdbFieldVarList object is a sequence listing all variables available for the current step
    and frame. Each item in the sequence is itself a sequence fully describing the given variable.

    .. note::
        This object can be accessed by::

            import visualization
            session.viewports[name].layers[name].odbDisplay.fieldVariables
            session.viewports[name].odbDisplay.fieldVariables
    """

    ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...BasicGeometry.Vertex import Vertex
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from .ConstrainedSketchConstraint import ConstrainedSketchConstraint


@abaqus_class_doc
class EqualDistanceConstraint(ConstrainedSketchConstraint):
    @abaqus_method_doc
    def __init__(self, entity1: str, entity2: ConstrainedSketchGeometry, midpoint: Vertex):
        """This method creates an equal distance constraint. This constraint can be applied between a midpoint
        ConstrainedSketchVertex object and any other two ConstrainedSketchVertex objects or between a midpoint
        ConstrainedSketchVertex object and two ConstrainedSketchGeometry objects that are lines. The equal
        distance constraint forces the midpoint vertex to remain at an equal distance from the two other
        vertices or lines.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].EqualDistanceConstraint

        Parameters
        ----------
        entity1
            AConstrainedSketchGeometry object specifying the first line or ConstrainedSketchVertex object.
        entity2
            A ConstrainedSketchGeometry object specifying the second line or ConstrainedSketchVertex object.
        midpoint
            A ConstrainedSketchVertex object specifying the vertex that will be positioned an equal distance from
            **entity1** and **entity2**.

        Returns
        -------
        ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .SectionPointArray import SectionPointArray


@abaqus_class_doc
class FieldLocation:
    """The FieldLocation object specifies locations for which data are available in the field. For example, a
    displacement field will have a FieldLocation object with a **position** member value of NODAL. The
    FieldLocation object has no constructor; it is created automatically as an element of the **location**
    member of a FieldOutput object by the addData method of a FieldOutput object.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].locations[i]
    """

    #: A SymbolicConstant specifying the position of the output in the element. Possible values
    #: are:NODAL, specifying the values calculated at the nodes.INTEGRATION_POINT, specifying
    #: the values calculated at the integration points.ELEMENT_NODAL, specifying the values
    #: obtained by extrapolating results calculated at the integration
    #: points.ELEMENT_FACE.CENTROID, specifying the value at the centroid obtained by
    #: extrapolating results calculated at the integration points.
    position: SymbolicConstant

    #: A SectionPointArray object.
    sectionPoints: SectionPointArray = []



from __future__ import annotations

from .FXComposite import FXComposite
from .FXWindow import FXWindow


class FXMenuSeparator(FXWindow):
    """The menu separator is a simple decorative groove used to delineate items in a popup menu."""

    def __init__(self, p: FXComposite, opts: int = 0):
        """Construct a menu separator.

        Parameters
        ----------
        p : FXComposite

        opts : int
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXWindow.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXWindow.
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .BoundaryConditionState import BoundaryConditionState


@abaqus_class_doc
class MaterialFlowBCState(BoundaryConditionState):
    """The MaterialFlowBCState object stores the propagating data for a connector material flow boundary
    condition in a step. One instance of this object is created internally by the MaterialFlowBC object for each
    step. The instance is also deleted internally by the MaterialFlowBC object. The MaterialFlowBCState object
    has no constructor or methods. The MaterialFlowBCState object is derived from the BoundaryConditionState
    object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BOUNDARY
    """

    #: A Float specifying the material flow magnitude.
    magnitude: float | None = None

    #: A SymbolicConstant specifying the propagation state of the material flow magnitude.
    #: Possible values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    magnitudeState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: SymbolicConstant

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from __future__ import annotations

from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXObject import FXObject
from .FXScrollArea import FXScrollArea

#: Text is NOT editable.
TEXT_READONLY: int = hash("TEXT_READONLY")

#: Wrap at word breaks.
TEXT_WORDWRAP: int = hash("TEXT_WORDWRAP")

#: Overstrike mode.
TEXT_OVERSTRIKE: int = hash("TEXT_OVERSTRIKE")

#: Fixed wrap columns.
TEXT_FIXEDWRAP: int = hash("TEXT_FIXEDWRAP")

#: Insert spaces for tabs.
TEXT_NO_TABS: int = hash("TEXT_NO_TABS")

#: Autoindent.
TEXT_AUTOINDENT: int = hash("TEXT_AUTOINDENT")

#: Show active line.
TEXT_SHOWACTIVE: int = hash("TEXT_SHOWACTIVE")

#: Select characters.
SELECT_CHARS: int = hash("SELECT_CHARS")

#: Select words.
SELECT_WORDS: int = hash("SELECT_WORDS")

#: Select lines.
SELECT_LINES: int = hash("SELECT_LINES")


class FXText(FXScrollArea):
    """Multiline text widget."""

    def __init__(
        self,
        p: FXComposite,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = 0,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Construct multi-line text widget.

        Parameters
        ----------
        p : FXComposite

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def appendText(self, text: str, n: int, notify: bool = False):
        """Append n characters of text at the end of the buffer.

        Parameters
        ----------
        text : str

        n : int

        notify : bool
        """

    def canFocus(self):
        """Returns True because a text widget can receive focus.

        Reimplemented from FXWindow.
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXComposite.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXComposite.
        """

    def disable(self):
        """Disable the text widget.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enable the text widget.

        Reimplemented from FXWindow.
        """

    def getBarColor(self):
        """Return bar color."""

    def getBarColumns(self):
        """Return number of columns used for line numbers."""

    def getChar(self, pos: int):
        """Get character at position in text buffer.

        Parameters
        ----------
        pos : int
        """

    def getContentHeight(self):
        """Get default height.

        Reimplemented from FXScrollArea.
        """

    def getContentWidth(self):
        """Get default width.

        Reimplemented from FXScrollArea.
        """

    def getCursorCol(self):
        """Return cursor row, i.e. indent position."""

    def getCursorColor(self):
        """Return cursor color."""

    def getCursorPos(self):
        """Return the cursor position."""

    def getCursorRow(self):
        """Return cursor row."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXScrollArea.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXScrollArea.
        """

    def getFont(self):
        """Return text font."""

    def getLength(self):
        """Return length of buffer."""

    def getMarginBottom(self):
        """Return bottom margin."""

    def getMarginLeft(self):
        """Return left margin."""

    def getMarginRight(self):
        """Return right margin."""

    def getMarginTop(self):
        """Return top margin."""

    def getNumberColor(self):
        """Return line number color."""

    def getPosAt(self, x: int, y: int):
        """Return text position at given visible x,y coordinate.

        Parameters
        ----------
        x : int

        y : int
        """

    def getSelBackColor(self):
        """Return selected background color."""

    def getSelEndPos(self):
        """Return selendpos."""

    def getSelStartPos(self):
        """Return selstartpos."""

    def getSelTextColor(self):
        """Return selected text color."""

    def getText(self, text: str, n: int):
        """Retrieve text into buffer.

        Parameters
        ----------
        text : str

        n : int
        """

    def getTextColor(self):
        """Return text color."""

    def getTopLine(self):
        """Return position of top line."""

    def getVisCols(self):
        """Return number of visible columns."""

    def getVisRows(self):
        """Return number of visible rows."""

    def insertText(self, pos: int, text: str, n: int, notify: bool = False):
        """Insert n characters of text at position pos into the buffer.

        Parameters
        ----------
        pos : int

        text : str

        n : int

        notify : bool
        """

    def isEditable(self):
        """Return True if text is editable."""

    def isModified(self):
        """Return True if text was modified."""

    def isPosSelected(self, pos: int):
        """Return True if position pos is selected.

        Parameters
        ----------
        pos : int
        """

    def killSelection(self, notify: bool = False):
        """Unselect the text.

        Parameters
        ----------
        notify : bool
        """

    def lineEnd(self, pos: int):
        """Return position of end of line containing position pos.

        Parameters
        ----------
        pos : int
        """

    def lineStart(self, pos: int):
        """Return position of begin of line containing position pos.

        Parameters
        ----------
        pos : int
        """

    def makePositionVisible(self, pos: int):
        """Scroll text to make the given position visible.

        Parameters
        ----------
        pos : int
        """

    def moveContents(self, x: int, y: int):
        """Scroll the contents. Reimplemented from FXScrollArea.

        Parameters
        ----------
        x : int

        y : int
        """

    def nextLine(self, pos: int, nl: int = 1):
        """Return start of next line.

        Parameters
        ----------
        pos : int

        nl : int
        """

    def nextRow(self, pos: int, nr: int = 1):
        """Return start of next row.

        Parameters
        ----------
        pos : int

        nr : int
        """

    def position(self, x: int, y: int, w: int, h: int):
        """Move and resize this window in the parent's coordinates. Reimplemented from FXWindow.

        Parameters
        ----------
        x : int

        y : int

        w : int

        h : int
        """

    def prevLine(self, pos: int, nl: int = 1):
        """Return start of previous line.

        Parameters
        ----------
        pos : int

        nl : int
        """

    def prevRow(self, pos: int, nr: int = 1):
        """Return start of previous row.

        Parameters
        ----------
        pos : int

        nr : int
        """

    def recalc(self):
        """Need to recalculate size.

        Reimplemented from FXWindow.
        """

    def removeText(self, pos: int, n: int, notify: bool = False):
        """Remove n characters of text at position pos from the buffer.

        Parameters
        ----------
        pos : int

        n : int

        notify : bool
        """

    def replaceText(self, pos: int, m: int, text: str, n: int, notify: bool = False):
        """Replace m characters at pos by n characters.

        Parameters
        ----------
        pos : int

        m : int

        text : str

        n : int

        notify : bool
        """

    def resize(self, w: int, h: int):
        """Resize this window to the specified width and height. Reimplemented from FXWindow.

        Parameters
        ----------
        w : int

        h : int
        """

    def rowEnd(self, pos: int):
        """Return row end.

        Parameters
        ----------
        pos : int
        """

    def rowStart(self, pos: int):
        """Return row start.

        Parameters
        ----------
        pos : int
        """

    def setBarColor(self, clr: FXColor):
        """Change bar color.

        Parameters
        ----------
        clr : FXColor
        """

    def setBarColumns(self, cols: int):
        """Change number of columns used for line numbers.

        Parameters
        ----------
        cols : int
        """

    def setBottomLine(self, pos: int):
        """Make line containing pos the bottom line.

        Parameters
        ----------
        pos : int
        """

    def setCursorCol(self, col: int, notify: bool = False):
        """Set cursor column.

        Parameters
        ----------
        col : int

        notify : bool
        """

    def setCursorColor(self, clr: FXColor):
        """Change cursor color.

        Parameters
        ----------
        clr : FXColor
        """

    def setCursorPos(self, pos: int, notify: bool = False):
        """Set the cursor position.

        Parameters
        ----------
        pos : int

        notify : bool
        """

    def setCursorRow(self, row: int, notify: bool = False):
        """Set cursor row.

        Parameters
        ----------
        row : int

        notify : bool
        """

    def setEditable(self, edit: bool = True):
        """Set editable flag.

        Parameters
        ----------
        edit : bool
        """

    def setFont(self, fnt: FXFont):
        """Change text font.

        Parameters
        ----------
        fnt : FXFont
        """

    def setMarginBottom(self, pb: int):
        """Change bottom margin.

        Parameters
        ----------
        pb : int
        """

    def setMarginLeft(self, pl: int):
        """Change left margin.

        Parameters
        ----------
        pl : int
        """

    def setMarginRight(self, pr: int):
        """Change right margin.

        Parameters
        ----------
        pr : int
        """

    def setMarginTop(self, pt: int):
        """Change top margin.

        Parameters
        ----------
        pt : int
        """

    def setModified(self, mod: bool = True):
        """Set modified flag.

        Parameters
        ----------
        mod : bool
        """

    def setNumberColor(self, clr: FXColor):
        """Change line number color.

        Parameters
        ----------
        clr : FXColor
        """

    def setSelBackColor(self, clr: FXColor):
        """Change selected background color.

        Parameters
        ----------
        clr : FXColor
        """

    def setSelection(self, pos: int, len: int, notify: bool = False):
        """Select len characters starting at given position pos.

        Parameters
        ----------
        pos : int

        len : int

        notify : bool
        """

    def setSelTextColor(self, clr: FXColor):
        """Change selected text color.

        Parameters
        ----------
        clr : FXColor
        """

    def setText(self, text: str, n: int, notify: bool = False):
        """Change the text in the buffer to new text.

        Parameters
        ----------
        text : str

        n : int

        notify : bool
        """

    def setTextColor(self, clr: FXColor):
        """Change text color.

        Parameters
        ----------
        clr : FXColor
        """

    def setTopLine(self, pos: int):
        """Make line containing pos the top line.

        Parameters
        ----------
        pos : int
        """

    def setVisCols(self, cols: int):
        """Change number of visible columns.

        Parameters
        ----------
        cols : int
        """

    def setVisRows(self, rows: int):
        """Change number of visible rows.

        Parameters
        ----------
        rows : int
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .CommandRegister import CommandRegister


@abaqus_class_doc
class RegisteredTuple(CommandRegister, tuple):
    """This class allows you to create a tuple that can be queried from the GUI and is capable of notifying the
    GUI when the contents of any of the tuple's members change. The RegisteredTuple object is derived from the
    CommandRegister object.

    .. note::
        This object can be accessed by::

            import customKernel
    """

    @abaqus_method_doc
    def __init__(self, tuple: tuple):
        """This method creates a RegisteredTuple object.

        .. note::
            This function can be accessed by::

                customKernel.RegisteredTuple

        Parameters
        ----------
        tuple
            A tuple of objects. These objects must be derived from the CommandRegister class.

        Returns
        -------
        RegisteredTuple
            A RegisteredTuple object.
        """
        super().__init__()

    @abaqus_method_doc
    def Methods(self):
        """The RegisteredTuple object supports the same methods as a standard Python list object."""
        # TODO: Implement this method
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, STEP, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Crack import Crack


@abaqus_class_doc
class DebondVCCT(Crack):
    """The DebondVCCT object defines the parameters needed to activate crack propagation using VCCT. The
    DebondVCCT object is derived from the Crack object.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].engineeringFeatures.cracks[name]
            import assembly
            mdb.models[name].rootAssembly.engineeringFeatures.cracks[name]

        The corresponding analysis keywords are:

        - DEBOND
    """

    #: A Boolean specifying whether the crack is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the DebondVCCT object is created.
    initiationStep: str

    #: A String specifying the name of the SurfaceToSurfaceContactStd object that defines the
    #: surface to surface interaction for the crack surfaces.
    surfToSurfInteraction: str

    #: A SymbolicConstant specifying whether the debond force between the two surfaces at the
    #: crack tip is to be released immediately or gradually during the following increment
    #: after debonding. Possible values are STEP and RAMP. The default value is STEP.
    debondingForceAmplitude: SymbolicConstant = STEP

    #: An Int specifying the frequency at which output will be printed to DAT file. The default
    #: value is 1.
    printToDATFrequency: int = 1

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        initiationStep: str,
        surfToSurfInteraction: str,
        debondingForceAmplitude: Literal[C.STEP, C.RAMP] = STEP,
        printToDATFrequency: int = 1,
    ):
        """This method creates a DebondVCCT object. Although the constructor is available both for parts and for
        the assembly, DebondVCCT objects are currently supported only under the assembly.

        .. note::
            This function can be accessed by::

                mdb.models[name].parts[name].engineeringFeatures.DebondVCCT
                mdb.models[name].rootAssembly.engineeringFeatures.DebondVCCT

        Parameters
        ----------
        name
            A String specifying the repository key.
        initiationStep
            A String specifying the name of the step in which the DebondVCCT object is created.
        surfToSurfInteraction
            A String specifying the name of the SurfaceToSurfaceContactStd object that defines the
            surface to surface interaction for the crack surfaces.
        debondingForceAmplitude
            A SymbolicConstant specifying whether the debond force between the two surfaces at the
            crack tip is to be released immediately or gradually during the following increment
            after debonding. Possible values are STEP and RAMP. The default value is STEP.
        printToDATFrequency
            An Int specifying the frequency at which output will be printed to DAT file. The default
            value is 1.

        Returns
        -------
        DebondVCCT
            A DebondVCCT object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        debondingForceAmplitude: Literal[C.STEP, C.RAMP] = STEP,
        printToDATFrequency: int = 1,
    ):
        """This method modifies the DebondVCCT object.

        Parameters
        ----------
        debondingForceAmplitude
            A SymbolicConstant specifying whether the debond force between the two surfaces at the
            crack tip is to be released immediately or gradually during the following increment
            after debonding. Possible values are STEP and RAMP. The default value is STEP.
        printToDATFrequency
            An Int specifying the frequency at which output will be printed to DAT file. The default
            value is 1.
        """
        ...
